"
 COPYRIGHT (c) 1996 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"



AbstractSourceCodeManager subclass:#NonVersioningSourceCodeManager
	instanceVariableNames:''
	classVariableNames:'SourceRoot'
	poolDictionaries:''
	category:'System-SourceCodeManagement'
!

!NonVersioningSourceCodeManager class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1996 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"


!

documentation
"
    Abstract superclass for SourceCodeManagers, which cannot do any
    versioning (i.e. access sources from a plain, unstructured file)

    [author:]
        Claus Gittinger

    [see also:]
        CVSSourceCodeManager
"


! !

!NonVersioningSourceCodeManager class methodsFor:'accessing'!

sourceDirectory
    "return the forced root.
     If no sourceRoot was given, a search along the systemPath
     for a sudirectory named 'source' is made"

    ^ SourceRoot

    "Created: 14.2.1997 / 21:03:47 / cg"
!

sourceDirectory:aDirectoryName
    "set the forced root.
     If no sourceRoot was given, a search along the systemPath
     for a sudirectory named 'source' is made.
     Otherwise, only that directory is investigated."

    SourceRoot := aDirectoryName

    "Created: 14.2.1997 / 21:03:56 / cg"
! !

!NonVersioningSourceCodeManager class methodsFor:'source code access'!

checkinClass:cls fileName:classFileName directory:packageDir module:moduleDir source:sourceFileName logMessage:logMessage force:force
    "enter a classes source code (which has been already filed out into sourceFileName)
     into the source repository. If the force argument is true, no merge is done;
     instead, the code is checked in as given (Dangerous).
     Return true if ok, false if not."

    self warn:'This SourceCodeManager only allows readAccess'.
    ^ false

    "Created: 14.2.1997 / 21:04:21 / cg"
!

streamForClass:aClass fileName:fileName revision:revision directory:packageDir module:moduleDir cache:cacheIt
    "extract a source file and return an open readStream on it,
     or nil if the extract failed. If revision is nil or (#newest), take
     the latest; otherwise, a specific revisions source is extracted.
     The cls argument is currently ignored, but may be used in future versions,
     to validate the correct container against the class."

    ^ self subclassResponsibility

    "Modified: 14.2.1997 / 20:52:19 / cg"
    "Created: 14.2.1997 / 21:04:45 / cg"
! !

!NonVersioningSourceCodeManager class methodsFor:'source code administration'!

checkForExistingContainerInModule:moduleDir package:packageDir container:fileName
    "check for a container to exist - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:13 / cg"
    "Modified: 14.2.1997 / 21:19:55 / cg"
!

checkForExistingModule:moduleDir
    "check for a module to exist - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:14 / cg"
    "Modified: 14.2.1997 / 21:19:59 / cg"
!

checkForExistingModule:moduleDir package:packageDir
    "check for a package to exist - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:14 / cg"
    "Modified: 14.2.1997 / 21:20:02 / cg"
!

createContainerFor:aClass inModule:moduleDir package:packageDir container:fileName
    "create a container - this does an initial checkin.
      - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:14 / cg"
    "Modified: 14.2.1997 / 21:20:15 / cg"
!

createModule:moduleName
    "create a module directory - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:14 / cg"
    "Modified: 14.2.1997 / 21:20:17 / cg"
!

createModule:moduleDirName package:packageDirName
    "create a package directory - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:15 / cg"
    "Modified: 14.2.1997 / 21:20:20 / cg"
!

initialRevisionStringFor:aClass inModule:moduleDir package:packageDir container:fileName
    "return a string usable as initial revision string.
     Here, a dummy is returned."

    "/ do not return a single Header string - RCS would expand it
    ^ '$' , 'Header:' , '$'

    "Created: 14.2.1997 / 21:22:20 / cg"
!

removeContainerFor:aClass inModule:moduleDir package:packageDir container:fileName
    "remove a container - not implemented here"

    ^ false

    "Created: 14.2.1997 / 21:05:15 / cg"
    "Modified: 14.2.1997 / 21:20:22 / cg"
!

revisionInfoFromString:aString
    "{ Pragma: +optSpace }"

    "return a dictionary filled with revision info.
     This extracts the relevant info from aString."

    ^ nil

    "Created: 29.1.1997 / 19:01:12 / cg"
!

revisionLogOf:cls fromRevision:rev1 toRevision:rev2 fileName:classFileName directory:packageDir module:moduleDir
    "return info about the repository container and
     (part of) the revisionlog as a collection of revision entries.
     Return nil on failure.
      - not implemented here"

    ^ nil

    "Created: 14.2.1997 / 21:05:15 / cg"
    "Modified: 14.2.1997 / 21:20:30 / cg"
!

writeHistoryLogSince:timeGoal filterSTSources:filter filterUser:userFilter to:aStream
    "send a full historyLog to some stream - not implemented here"

    aStream nextPutAll:'This sourceCodeManager cannot extract history logs'.
    ^ nil

    "Created: 14.2.1997 / 21:05:15 / cg"
    "Modified: 14.2.1997 / 21:20:33 / cg"
! !

!NonVersioningSourceCodeManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic3/Attic/NonVersioningSourceCodeManager.st,v 1.3 1997-01-29 18:31:15 cg Exp $'
! !
